@@ -6,6 +6,8 @@
 #undef USE_SOCKETS_AS_HANDLES
 #include <XSUB.h>
 
+#include "c_module.h"
+
 #undef free
 #undef malloc
 #undef realloc
@@ -20,8 +22,6 @@
 #ifdef _WIN32
 
 #include <initguid.h>
-#include <winsock2.h>
-#include <ws2tcpip.h>
 #include <tchar.h>
 #include <io.h>
 /*
@@ -32,13 +32,6 @@
 
 #include <stdlib.h>
 #include <unistd.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <sys/un.h>
-#include <netinet/in.h>
-#include <netinet/tcp.h>
-#include <arpa/inet.h>
-#include <netdb.h>
 
 #endif
 
@@ -101,19 +94,9 @@ EXTERN int my_debug( const char *fmt, ... );
 typedef unsigned short			uint16_t;
 typedef unsigned char			uint8_t;
 typedef unsigned short			sa_family_t;
-#else
-/*
-typedef unsigned long			u_long;
-*/
 #endif
 
 /* removing from perlio */
-/*
-#undef htonl
-#undef htons
-#undef ntohl
-#undef ntohs
-*/
 #undef accept
 #undef bind
 #undef connect
@@ -158,18 +141,88 @@ typedef unsigned long			u_long;
 
 /* removing from perl */
 #undef Newx
+#undef Newxz
+#undef Safefree
+#undef Renew
+#undef Copy
+
+#if SC_DEBUG > 1
+
+extern HV				*hv_dbg_mem;
+extern perl_mutex		dbg_mem_lock;
+
+void debug_init();
+void debug_free();
+
+#define Newx(v,n,t) { \
+	char __v[41], __msg[128]; \
+	MUTEX_LOCK( &dbg_mem_lock ); \
+	(v) = ((t*) safemalloc( (size_t) (n) * sizeof(t) )); \
+	sprintf( __v, "0x%lx", (size_t) (v) ); \
+	sprintf( __msg, "0x%lx malloc(%lu * %lu) called at %s:%d", \
+		(size_t) (v), (size_t) (n), sizeof(t), __FILE__, __LINE__ ); \
+	_debug( "%s\n", __msg ); \
+	(void) hv_store( hv_dbg_mem, \
+		__v, (I32) strlen( __v ), newSVpvn( __msg, strlen( __msg ) ), 0 ); \
+	MUTEX_UNLOCK( &dbg_mem_lock ); \
+}
+
+#define Newxz(v,n,t) { \
+	char __v[41], __msg[128]; \
+	MUTEX_LOCK( &dbg_mem_lock ); \
+	(v) = ((t*) safecalloc( (size_t) (n), sizeof(t) )); \
+	sprintf( __v, "0x%lx", (size_t) (v) ); \
+	sprintf( __msg, "0x%lx calloc(%lu * %lu) called at %s:%d", \
+		(size_t) (v), (size_t) (n), sizeof(t), __FILE__, __LINE__ ); \
+	_debug( "%s\n", __msg ); \
+	(void) hv_store( hv_dbg_mem, \
+		__v, (I32) strlen( __v ), newSVpvn( __msg, strlen( __msg ) ), 0 ); \
+	MUTEX_UNLOCK( &dbg_mem_lock ); \
+}
+
+#define Safefree(x) { \
+	char __v[41]; \
+	MUTEX_LOCK( &dbg_mem_lock ); \
+	if( (x) != NULL ) { \
+		sprintf( __v, "0x%lx", (size_t) (x) ); \
+		_debug( "0x%lx free() called at %s:%d\n", \
+			(size_t) (x), __FILE__, __LINE__ ); \
+		(void) hv_delete( hv_dbg_mem, __v, (I32) strlen( __v ), G_DISCARD ); \
+		safefree( (x) ); (x) = NULL; \
+	} \
+	MUTEX_UNLOCK( &dbg_mem_lock ); \
+}
+
+#define Renew(v,n,t) { \
+	register void *__p = (v); \
+	char __v[41], __msg[128]; \
+	MUTEX_LOCK( &dbg_mem_lock ); \
+	sprintf( __v, "0x%lx", (size_t) (v) ); \
+	(void) hv_delete( hv_dbg_mem, __v, (I32) strlen( __v ), G_DISCARD ); \
+	(v) = ((t*) saferealloc( __p, (size_t) (n) * sizeof(t) )); \
+	sprintf( __v, "0x%lx", (size_t) (v) ); \
+	sprintf( __msg, "0x%lx realloc(0x%lx, %lu * %lu) called at %s:%d", \
+		(size_t) (v), (size_t) __p, (size_t) (n), sizeof(t), \
+		__FILE__, __LINE__ ); \
+	_debug( "%s\n", __msg ); \
+	(void) hv_store( hv_dbg_mem, \
+		__v, (I32) strlen( __v ), newSVpvn( __msg, strlen( __msg ) ), 0 ); \
+	MUTEX_UNLOCK( &dbg_mem_lock ); \
+}
+
+#else
+
 #define Newx(v,c,t) \
 	( (v) = ( (t*) malloc( (c) * sizeof(t) ) ) )
-#undef Newxz
 #define Newxz(v,c,t) \
 	( (v) = ( (t*) calloc( (c), sizeof(t) ) ) )
-#undef Safefree
 #define Safefree(x) \
 	if( (x) != NULL ) { free( (x) ); (x) = NULL; }
-#undef Renew
 #define Renew(v,n,t) \
 	( (v) = ( (t*) realloc( (void *) (v), (n) * sizeof(t) ) ) )
-#undef Copy
+
+#endif
+
 #define Copy(s,d,n,t) \
 	( memcpy( (char*)(d), (const char*)(s), (n) * sizeof(t) ) )
 
@@ -180,26 +233,17 @@ typedef unsigned long			u_long;
 #define EINPROGRESS				WSAEINPROGRESS
 #define ETIMEDOUT				WSAETIMEDOUT
 
-#ifndef AF_BLUETOOTH
-#define AF_BLUETOOTH			32
-#endif
-
 struct sockaddr_un {
 	sa_family_t					sun_family;				/* AF_UNIX */
 	char						sun_path[108];			/* pathname */
 };
 
-#else
+#else /* POSIX */
 
-#define SOCKET					int
 #define SOCKET_ERROR			-1
 #define INVALID_SOCKET			-1
 #define ESOCKETBROKEN			1111
 
-#ifndef AF_BLUETOOTH
-#define AF_BLUETOOTH			31
-#endif
-
 #ifndef AF_INET6
 #define SC_OLDNET				1
 #define AF_INET6				23
@@ -213,49 +257,15 @@ struct sockaddr_in6 {
 	struct in6_addr				sin6_addr;		/* IPv6 address. */
 	uint32_t					sin6_scope_id;	/* Set of interfaces for a scope. */
 };
-#endif
+#endif /* ! AF_INET6 */
 
-#endif
-
-#ifndef NI_MAXHOST
-#define NI_MAXHOST				1025
-#endif
-#ifndef NI_MAXSERV
-#define NI_MAXSERV				32
-#endif
-#ifndef SOCK_RDM
-#define SOCK_RDM				4
-#endif
-#ifndef SOCK_SEQPACKET
-#define SOCK_SEQPACKET			5
-#endif
-#ifndef AF_IRDA
-#define AF_IRDA					26
-#endif
+#endif /* POSIX */
 
 #undef MAX
 #define MAX(x,y) ( (x) < (y) ? (y) : (x) )
 #undef MIN
 #define MIN(x,y) ( (x) < (y) ? (x) : (y) )
 
-#ifdef _WIN32
-#define BTPROTO_RFCOMM			0x0003
-#define BTPROTO_L2CAP			0x0100
-#else
-#ifndef SC_HAS_BLUETOOTH
-#define BTPROTO_RFCOMM			3
-#define BTPROTO_L2CAP			0
-#endif
-#endif
-
-#define SOCK_STATE_INIT			0
-#define SOCK_STATE_BOUND		1
-#define SOCK_STATE_LISTEN		2
-#define SOCK_STATE_CONNECTED	3
-#define SOCK_STATE_SHUTDOWN		4
-#define SOCK_STATE_CLOSED		5
-#define SOCK_STATE_ERROR		99
-
 #define ADDRUSE_CONNECT			0
 #define ADDRUSE_LISTEN			1
 
@@ -303,17 +313,12 @@ typedef struct st_sockaddr_l2	SOCKADDR_L2CAP;
 
 #endif
 
-#define SOCKADDR_SIZE_MAX		128
-
-typedef struct st_my_sockaddr {
-	socklen_t					l;
-	char						a[SOCKADDR_SIZE_MAX];
-} my_sockaddr_t;
+typedef struct st_sc_sockaddr	my_sockaddr_t;
 
 #define MYSASIZE(sa)			((sa).l + sizeof(socklen_t))
 
-typedef struct st_my_thread_var {
-	struct st_my_thread_var		*prev, *next;
+typedef struct st_socket_class {
+	struct st_socket_class		*prev, *next;
 	unsigned long				id;
 	SOCKET						sock;
 	int							s_domain;
@@ -332,13 +337,13 @@ typedef struct st_my_thread_var {
 #ifdef USE_ITHREADS
 	perl_mutex					thread_lock;
 #endif
-} my_thread_var_t;
+} socket_class_t;
 
-#define SC_TV_CASCADE			31
+#define SC_CASCADE				31
 
-typedef struct st_my_global {
-	my_thread_var_t				*first_thread[SC_TV_CASCADE];
-	my_thread_var_t				*last_thread[SC_TV_CASCADE];
+typedef struct st_sc_global {
+	socket_class_t				*first_socket[SC_CASCADE];
+	socket_class_t				*last_socket[SC_CASCADE];
 	long						last_errno;
 	char						last_error[256];
 	int							destroyed;
@@ -346,68 +351,72 @@ typedef struct st_my_global {
 #ifdef USE_ITHREADS
 	perl_mutex					thread_lock;
 #endif
-} my_global_t;
+} sc_global_t;
 
-extern my_global_t global;
+extern sc_global_t global;
 
 #ifdef USE_ITHREADS
 
 #define GLOBAL_LOCK()			MUTEX_LOCK( &global.thread_lock )
 #define GLOBAL_UNLOCK()			MUTEX_UNLOCK( &global.thread_lock )
-#define TV_LOCK(tv)				MUTEX_LOCK( &tv->thread_lock )
-#define TV_UNLOCK(tv)			MUTEX_UNLOCK( &tv->thread_lock )
+#define SOCK_LOCK(tv)				MUTEX_LOCK( &tv->thread_lock )
+#define SOCK_UNLOCK(tv)			MUTEX_UNLOCK( &tv->thread_lock )
 
 #else
 
 #define GLOBAL_LOCK()
 #define GLOBAL_UNLOCK()
-#define TV_LOCK(tv)
-#define TV_UNLOCK(tv)
+#define SOCK_LOCK(tv)
+#define SOCK_UNLOCK(tv)
 
 #endif
 
-#define TV_ERRNOLAST(tv) \
-	(tv)->last_error[0] = '\0'; \
-	(tv)->last_errno = Socket_errno()
+#define SOCK_ERRNOLAST(tv) \
+	SOCK_ERRNO(tv, Socket_errno())
 
-#define TV_ERRNO(tv,code) \
-	(tv)->last_error[0] = '\0'; \
-	(tv)->last_errno = code
+#define SOCK_ERRNO(tv,code) \
+	do { \
+		(tv)->last_error[0] = '\0'; \
+		(tv)->last_errno = code; \
+	} while( 0 )
 
-#define TV_ERROR(tv,str) \
-	my_strncpy( (tv)->last_error, str, sizeof( (tv)->last_error ) ); \
-	(tv)->last_errno = -1
+#define SOCK_ERROR(tv,str) \
+	do { \
+		my_strncpy( (tv)->last_error, str, sizeof( (tv)->last_error ) ); \
+		(tv)->last_errno = -1; \
+	} while( 0 )
 
-#define GLOBAL_ERROR(code,str) { \
-	SV *__sv = get_sv( "!", TRUE ); \
-	global.last_errno = code; \
-	strncpy( global.last_error, str, sizeof(global.last_error) ); \
-	sv_setiv( __sv, (IV) code ); \
-	sv_setpv( __sv, global.last_error ); \
-	SvIOK_on( __sv ); \
-}
+#define GLOBAL_ERROR(code,str) \
+	do { \
+		SV *__sv = get_sv( "!", TRUE ); \
+		global.last_errno = code; \
+		strncpy( global.last_error, str, sizeof(global.last_error) ); \
+		sv_setiv( __sv, (IV) code ); \
+		sv_setpv( __sv, global.last_error ); \
+		SvIOK_on( __sv ); \
+	} while( 0 )
 
-#define GLOBAL_ERRNO(code) { \
-	SV *__sv = get_sv( "!", TRUE ); \
-	char __s[255]; \
-	global.last_errno = code; \
-	Socket_error( __s, sizeof(__s), code ); \
-	sv_setiv( __sv, (IV) code ); \
-	sv_setpv( __sv, __s ); \
-	SvIOK_on( __sv ); \
-}
+#define GLOBAL_ERRNO(code) \
+	do { \
+		SV *__sv = get_sv( "!", TRUE ); \
+		char __s[255]; \
+		global.last_errno = code; \
+		Socket_error( __s, sizeof(__s), code ); \
+		sv_setiv( __sv, (IV) code ); \
+		sv_setpv( __sv, __s ); \
+		SvIOK_on( __sv ); \
+	} while( 0 )
 
 #define GLOBAL_ERRNOLAST()	GLOBAL_ERRNO(Socket_errno())
 
-EXTERN void my_thread_var_add( my_thread_var_t *tv );
-EXTERN void my_thread_var_rem( my_thread_var_t *tv );
-EXTERN void my_thread_var_free( my_thread_var_t *tv );
-EXTERN my_thread_var_t *my_thread_var_find( SV *sv );
+EXTERN void socket_class_add( socket_class_t *sc );
+EXTERN void socket_class_rem( socket_class_t *sc );
+EXTERN void socket_class_free( socket_class_t *sc );
+EXTERN socket_class_t *socket_class_find( SV *sv );
 
 EXTERN char *my_itoa( char *str, long value, int radix );
 EXTERN char *my_strncpy( char *dst, const char *src, size_t len );
 EXTERN char *my_strcpy( char *dst, const char *src );
-EXTERN char *my_strncpyu( char *dst, const char *src, size_t len );
 EXTERN int my_stricmp( const char *cs, const char *ct );
 
 #ifdef _WIN32
@@ -434,18 +443,14 @@ EXTERN int inet_aton( const char *cp, struct in_addr *inp );
 
 EXTERN void Socket_setaddr_UNIX( my_sockaddr_t *addr, const char *path );
 EXTERN int Socket_setaddr_INET(
-	my_thread_var_t *tv, const char *host, const char *port, int use );
+	socket_class_t *sc, const char *host, const char *port, int use );
 EXTERN int Socket_setaddr_BTH(
-	my_thread_var_t *tv, const char *host, const char *port, int use );
+	socket_class_t *sc, const char *host, const char *port, int use );
 EXTERN int Socket_setblocking( SOCKET s, int value );
-EXTERN int Socket_setopt(
-	SV *this, int level, int optname, const void *optval, socklen_t optlen );
-EXTERN int Socket_getopt(
-	SV *this, int level, int optname, void *optval, socklen_t *optlen );
 EXTERN int Socket_domainbyname( const char *name );
 EXTERN int Socket_typebyname( const char *name );
 EXTERN int Socket_protobyname( const char *name );
-EXTERN int Socket_write( SV *this, const char *buf, size_t len );
+EXTERN int Socket_write( socket_class_t *sc, const char *buf, int len );
 EXTERN void Socket_error( char *str, DWORD len, long num );
 
 #define IPPORT4(ip,port) \
@@ -471,11 +476,6 @@ EXTERN void Socket_error( char *str, DWORD len, long num );
 EXTERN int my_ba2str( const bdaddr_t *ba, char *str );
 EXTERN int my_str2ba( const char *str, bdaddr_t *ba );
 
-/*
-EXTERN unsigned short my_htons( unsigned short a );
-EXTERN unsigned short my_ntohs( unsigned short a );
-*/
-
 #ifdef SC_HAS_BLUETOOTH
 EXTERN void boot_Socket__Class__BT();
 #endif
